import path from 'path';
import { fs } from '@modern-js/utils';
import { types as t } from '@babel/core';
import { isProjectFile } from "./utils";
const replaceValueHash = {};

const isResoureInSrc = (srcDir, resourecPath) => !path.posix.relative(srcDir, resourecPath).includes('..');

const getImportFileDistPath = (compilerFile, srcDir, importName) => {
  const dir = path.posix.dirname(compilerFile);
  const compilerFileRelativeLoc = path.posix.relative(dir, srcDir);
  const importFileRelativeLoc = path.posix.relative(srcDir, path.posix.join(dir, importName));
  const inSrc = isResoureInSrc(srcDir, path.posix.join(dir, importName));
  const importFileDistPath = path.posix.join(inSrc ? '..' : '../..', compilerFileRelativeLoc, 'styles', importFileRelativeLoc);
  return importFileDistPath;
};

export const isStaticFile = (file, filename) => {
  const tests = [[/\.js$/, '.js'], [/\.jsx$/, '.jsx'], [/\.ts$/, '.ts'], [/\.tsx$/, '.tsx']]; // check this file is static file
  // by string and determine if the file with the added suffix exists

  return !tests.some(([regex, prefix]) => regex.test(file) || fs.existsSync(path.join(path.dirname(filename), file) + prefix) || fs.existsSync(path.join(path.dirname(filename), file, 'index') + prefix));
};

const isStyleFile = file => {
  const tests = [/\.css$/, /\.less$/, /\.sass$/, /\.scss$/];
  return tests.some(regex => regex.test(file));
};

const getReplacePath = (importName, filename, srcDir, importStyle) => {
  if (!filename || !importName) {
    return '';
  }

  if (!isProjectFile(importName)) {
    return '';
  }

  if (!isStaticFile(importName, filename)) {
    return '';
  }

  let realFilepath = getImportFileDistPath(filename, srcDir, importName);

  if (isStyleFile(realFilepath) && importStyle === 'compiled-code') {
    realFilepath = realFilepath.replace(/\.(less|sass|scss)$/, '.css');
  }

  return realFilepath;
};

const importPath = () => ({
  name: 'import-path',
  visitor: {
    Program(nodePath, {
      opts,
      file
    }) {
      nodePath.traverse({
        ImportDeclaration({
          node
        }) {
          const {
            source
          } = node;
          const {
            appDirectory,
            importStyle = 'source-code'
          } = opts;
          const srcDir = `${appDirectory}/src`;
          const {
            opts: {
              filename
            }
          } = file;
          const importName = source !== null && source !== void 0 && source.value ? source.value : undefined;
          const replaceValue = getReplacePath(importName, filename, srcDir, importStyle);
          const hashKey = filename + (importName || '');

          if (replaceValue) {
            if (typeof filename === 'string' && !replaceValueHash[hashKey]) {
              node.source.value = replaceValue;
              replaceValueHash[hashKey] = replaceValue;
            } else if (typeof filename === 'string' && replaceValueHash[hashKey]) {
              node.source.value = replaceValueHash[hashKey];
            } else {
              node.source.value = replaceValue;
            }
          }
        },

        // dynamic import
        CallExpression({
          node
        }) {
          const {
            appDirectory,
            importStyle = 'source-code'
          } = opts;
          const srcDir = `${appDirectory}/src`;
          const {
            filename
          } = file.opts;
          const {
            callee,
            arguments: args
          } = node;

          if (callee.type === 'Import' || callee.type === 'Identifier' && callee.name === 'require') {
            const firstArg = args[0];

            if (firstArg.value) {
              const importName = firstArg.value;
              const replaceValue = getReplacePath(importName, filename, srcDir, importStyle);
              const hashKey = filename + (importName || '');

              if (replaceValue) {
                if (typeof filename === 'string' && !replaceValueHash[hashKey]) {
                  node.arguments = [t.stringLiteral(replaceValue)];
                  replaceValueHash[hashKey] = replaceValue;
                } else if (typeof filename === 'string' && replaceValueHash[hashKey]) {
                  node.arguments = [t.stringLiteral(replaceValueHash[hashKey])];
                } else {
                  node.arguments = [t.stringLiteral(replaceValue)];
                }
              }
            }
          }
        }

      });
    }

  }
});

export default importPath;